@@ -9,13 +9,15 @@ import collections
 import json
 import abc
 import re
+import numpy as np
 
 # gemini imports
 import gemini_utils as util
 from gemini_constants import *
-from gemini_utils import OrderedSet, OrderedDict, itersubclasses
+from gemini_utils import OrderedSet, OrderedDict, itersubclasses, partition
 import compression
 from sql_utils import ensure_columns, get_select_cols_and_rest
+from gemini_subjects import get_subjects
 
 
 class RowFormat:
@@ -35,7 +37,7 @@ class RowFormat:
     def format(self, row):
         """ return a string representation of a GeminiRow object
         """
-        return '\t'.join([str(row[c]) for c in row])
+        return '\t'.join([str(row.row[c]) for c in row.row])
 
     @abc.abstractmethod
     def format_query(self, query):
@@ -50,37 +52,116 @@ class RowFormat:
         return True if there is no additional predicate"""
         return True
 
+    @abc.abstractmethod
+    def header(self, fields):
+        """ return a header for the row """
+        return "\t".join(fields)
+
 
 class DefaultRowFormat(RowFormat):
 
     name = "default"
 
-    @classmethod
+    def __init__(self, args):
+        pass
+
     def format(self, row):
-        return '\t'.join([str(row[c]) for c in row])
+        return '\t'.join([str(row.row[c]) for c in row.row])
 
-    @classmethod
     def format_query(self, query):
         return query
 
-    @classmethod
     def predicate(self, row):
         return True
 
+    def header(self, fields):
+        """ return a header for the row """
+        return "\t".join(fields)
+
+class CarrierSummary(RowFormat):
+    """
+    Generates a count of the carrier/noncarrier status of each feature in a given
+    column of the sample table
+
+    Assumes None == unknown.
+    """
+    name = "carrier_summary"
+
+    def __init__(self, args):
+        subjects = get_subjects(args)
+        self.carrier_summary = args.carrier_summary
+
+        # get the list of all possible values in the column
+        # but don't include None, since we are treating that as unknown.
+        self.column_types = list(set([getattr(x, self.carrier_summary)
+                                      for x in subjects.values()]))
+        self.column_types = [i for i in self.column_types if i is not None]
+        self.column_counters = {None: set()}
+        for ct in self.column_types:
+            self.column_counters[ct] = set([k for (k, v) in subjects.items() if
+                                            getattr(v, self.carrier_summary) == ct])
+
+
+    def format(self, row):
+        have_variant = set(row.variant_samples)
+        have_reference = set(row.HOM_REF_samples)
+        unknown = len(set(row.UNKNOWN_samples).union(self.column_counters[None]))
+        carrier_counts = []
+        for ct in self.column_types:
+            counts = len(self.column_counters[ct].intersection(have_variant))
+            carrier_counts.append(counts)
+        for ct in self.column_types:
+            counts = len(self.column_counters[ct].intersection(have_reference))
+            carrier_counts.append(counts)
+
+        carrier_counts.append(unknown)
+        carrier_counts = map(str, carrier_counts)
+        return '\t'.join([str(row.row[c]) for c in row.row] + carrier_counts)
+
+    def format_query(self, query):
+        return query
+
+    def predicate(self, row):
+        return True
+
+    def header(self, fields):
+        """ return a header for the row """
+        header_columns = self.column_types
+        if self.carrier_summary == "affected":
+            header_columns = self._rename_affected()
+        carriers = [x + "_carrier" for x in map(str, header_columns)]
+        noncarriers = [ x + "_noncarrier" for x in map(str, header_columns)]
+        fields += carriers
+        fields += noncarriers
+        fields += ["unknown"]
+        return "\t".join(fields)
+
+    def _rename_affected(self):
+        header_columns = []
+        for ct in self.column_types:
+            if ct == True:
+                header_columns.append("affected")
+            elif ct == False:
+                header_columns.append("unaffected")
+        return header_columns
+
+
 
 class TPEDRowFormat(RowFormat):
 
     name = "tped"
     NULL_GENOTYPES = ["."]
 
-    @classmethod
+    def __init__(self, args):
+        pass
+
     def format(self, row):
         VALID_CHROMOSOMES = map(str, range(1, 23)) + ["X", "Y", "XY", "MT"]
         chrom = row['chrom'].split("chr")[1]
         chrom = chrom if chrom in VALID_CHROMOSOMES else "0"
-        start = str(row['start'])
-        end = str(row['end'])
-        geno = [re.split('\||/', x) for x in row['gts'].split(",")]
+        start = str(row.row['start'])
+        end = str(row.row['end'])
+        geno = [re.split('\||/', x) for x in row.row['gts'].split(",")]
         geno = [["0", "0"] if any([y in self.NULL_GENOTYPES for y in x])
                 else x for x in geno]
         genotypes = " ".join(list(flatten(geno)))
@@ -88,37 +169,41 @@ class TPEDRowFormat(RowFormat):
         name = chrom + ":" +  start + "-" + end + ":" + alleles
         return " ".join([chrom, name, "0", start, genotypes])
 
-    @classmethod
     def format_query(self, query):
         NEED_COLUMNS = ["chrom", "rs_ids", "start", "gts", "type", "variant_id"]
         return ensure_columns(query, NEED_COLUMNS)
 
-    @classmethod
     def predicate(self, row):
         geno = [re.split("\||/", x) for x in row['gts']]
         geno = list(flatten(geno))
         num_alleles = len(set(geno).difference(self.NULL_GENOTYPES))
         return num_alleles > 0 and num_alleles <= 2 and row['type'] != "sv"
 
+    def header(self, fields):
+        return None
+
 
 class JSONRowFormat(RowFormat):
 
     name = "json"
 
-    @classmethod
+    def __init__(self, args):
+        pass
+
     def format(self, row):
         """Emit a JSON representation of a given row
         """
-        return json.dumps(row)
+        return json.dumps(row.row)
 
-    @classmethod
     def format_query(self, query):
         return query
 
-    @classmethod
     def predicate(self, row):
         return True
 
+    def header(self, fields):
+        return None
+
 
 class GeminiRow(object):
 
@@ -127,7 +212,8 @@ class GeminiRow(object):
                  gt_ref_depths=None, gt_alt_depths=None,
                  gt_quals=None, variant_samples=None,
                  HET_samples=None, HOM_ALT_samples=None,
-                 formatter=DefaultRowFormat):
+                 HOM_REF_samples=None, UNKNOWN_samples=None,
+                 formatter=DefaultRowFormat(None)):
         self.row = row
         self.gts = gts
         self.gt_types = gt_types
@@ -143,6 +229,8 @@ class GeminiRow(object):
         self.variant_samples = variant_samples
         self.HET_samples = HET_samples
         self.HOM_ALT_samples = HOM_ALT_samples
+        self.HOM_REF_samples = HOM_REF_samples
+        self.UNKNOWN_samples = UNKNOWN_samples
 
     def __getitem__(self, val):
         if val not in self.gt_cols:
@@ -154,7 +242,7 @@ class GeminiRow(object):
         return self
 
     def __repr__(self):
-        return self.formatter.format(self.row)
+        return self.formatter.format(self)
 
     def next(self):
         try:
@@ -225,7 +313,8 @@ class GeminiQuery(object):
             print row, gts[idx]
     """
 
-    def __init__(self, db, include_gt_cols=False, out_format="default"):
+    def __init__(self, db, include_gt_cols=False,
+                 out_format=DefaultRowFormat(None)):
         assert os.path.exists(db), "%s does not exist." % db
 
         self.db = db
@@ -235,30 +324,20 @@ class GeminiQuery(object):
 
         self._connect_to_database()
         # map sample names to indices. e.g. self.sample_to_idx[NA20814] -> 323
-        self.sample_to_idx = util.map_samples_to_indicies(self.c)
+        self.sample_to_idx = util.map_samples_to_indices(self.c)
         # and vice versa. e.g., self.idx_to_sample[323] ->  NA20814
-        self.idx_to_sample = util.map_indicies_to_samples(self.c)
-        self.formatter = self._set_formatter(out_format.lower())
+        self.idx_to_sample = util.map_indices_to_samples(self.c)
+        self.idx_to_sample_object = util.map_indices_to_sample_objects(self.c)
+        self.formatter = out_format
         self.predicates = [self.formatter.predicate]
 
 
-    def _set_formatter(self, out_format):
-        SUPPORTED_FORMATS = {x.name.lower(): x for x in itersubclasses(RowFormat)}
-
-        if not out_format in SUPPORTED_FORMATS:
-            raise NotImplementedError("Conversion to %s not supported. Valid "
-                                      "formats are %s."
-                                      % (out_format, SUPPORTED_FORMATS))
-        else:
-            return SUPPORTED_FORMATS[out_format]
-
-
     def _set_gemini_browser(self, for_browser):
         self.for_browser = for_browser
 
     def run(self, query, gt_filter=None, show_variant_samples=False,
             variant_samples_delim=',', predicates=None,
-            needs_genotypes=False):
+            needs_genotypes=False, show_families=False):
         """
         Execute a query against a Gemini database. The user may
         specify:
@@ -270,6 +349,8 @@ class GeminiQuery(object):
         self.gt_filter = gt_filter
         self.show_variant_samples = show_variant_samples
         self.variant_samples_delim = variant_samples_delim
+        self.needs_genotypes = needs_genotypes
+        self.show_families = show_families
         if predicates:
             self.predicates += predicates
 
@@ -301,9 +382,6 @@ class GeminiQuery(object):
         Return a header describing the columns that
         were selected in the query issued to a GeminiQuery object.
         """
-        if self.formatter.name in ["json"]:
-            return None
-
         if self.query_type == "no-genotypes":
             h = [col for col in self.all_query_cols]
         else:
@@ -312,8 +390,9 @@ class GeminiQuery(object):
                  - OrderedSet(self.select_columns)]
         if self.show_variant_samples:
             h += ["variant_samples", "HET_samples", "HOM_ALT_samples"]
-        return GeminiRow(OrderedDict(itertools.izip(h, h)),
-                         formatter=self.formatter)
+        if self.show_families:
+            h += ["families"]
+        return self.formatter.header(h)
 
     @property
     def sample2index(self):
@@ -356,6 +435,7 @@ class GeminiQuery(object):
             try:
                 row = self.c.next()
             except Exception as e:
+                self.conn.close()
                 raise StopIteration
             gts = None
             gt_types = None
@@ -367,6 +447,9 @@ class GeminiQuery(object):
             variant_names = []
             het_names = []
             hom_alt_names = []
+            hom_ref_names = []
+            unknown_names = []
+
 
             if self._query_needs_genotype_info():
                 gts = compression.unpack_genotype_blob(row['gts'])
@@ -389,6 +472,12 @@ class GeminiQuery(object):
                 het_names = [self.idx_to_sample[x] for x in het_samples]
                 hom_alt_samples = [x for x, y in enumerate(gt_types) if y == HOM_ALT]
                 hom_alt_names = [self.idx_to_sample[x] for x in hom_alt_samples]
+                hom_ref_samples = [x for x, y in enumerate(gt_types) if y == HOM_REF]
+                hom_ref_names = [self.idx_to_sample[x] for x in hom_ref_samples]
+                unknown_samples = [x for x, y in enumerate(gt_types) if y == UNKNOWN]
+                unknown_names = [self.idx_to_sample[x] for x in unknown_samples]
+                families = map(str, list(set([self.idx_to_sample_object[x].family_id
+                            for x in variant_samples])))
 
                 # skip the record if it does not meet the user's genotype filter
                 if self.gt_filter and not eval(self.gt_filter):
@@ -406,7 +495,13 @@ class GeminiQuery(object):
                     # e.g. replace gts[1085] with gts.NA20814
                     if '[' in col:
                         orig_col = self.gt_idx_to_name_map[col]
-                        fields[orig_col] = eval(col.strip())
+                        val = eval(col.strip())
+                        if type(val) in [np.int8, np.int32, np.bool_]:
+                            fields[orig_col] = int(val)
+                        elif type(val) in [np.float32]:
+                            fields[orig_col] = float(val)
+                        else:
+                            fields[orig_col] = val
                     else:
                         # asked for "gts" or "gt_types", e.g.
                         if col == "gts":
@@ -437,10 +532,13 @@ class GeminiQuery(object):
                     self.variant_samples_delim.join(het_names)
                 fields["HOM_ALT_samples"] = \
                     self.variant_samples_delim.join(hom_alt_names)
+            if self.show_families:
+                fields["families"] = self.variant_samples_delim.join(families)
 
             gemini_row = GeminiRow(fields, gts, gt_types, gt_phases,
                                    gt_depths, gt_ref_depths, gt_alt_depths,
                                    gt_quals, variant_names, het_names, hom_alt_names,
+                                   hom_ref_names, unknown_names,
                                    formatter=self.formatter)
 
             if not all([predicate(gemini_row) for predicate in self.predicates]):
@@ -643,7 +741,22 @@ class GeminiQuery(object):
         requested_genotype = "variants" in tokens and any([x.startswith("gt") for x in tokens])
         return requested_genotype or \
                self.include_gt_cols or \
-               self.show_variant_samples
+               self.show_variant_samples or self.needs_genotypes
+
+def select_formatter(args):
+    SUPPORTED_FORMATS = {x.name.lower(): x for x in
+                         itersubclasses(RowFormat)}
+
+    if hasattr(args, 'carrier_summary') and args.carrier_summary:
+        return SUPPORTED_FORMATS["carrier_summary"](args)
+
+    if not args.format in SUPPORTED_FORMATS:
+        raise NotImplementedError("Conversion to %s not supported. Valid "
+                                  "formats are %s."
+                                  % (args.format, SUPPORTED_FORMATS))
+    else:
+        return SUPPORTED_FORMATS[args.format](args)
+
 
 def flatten(l):
     """
