@@ -16,14 +16,11 @@ def load(parser, args):
     if (args.db is None or args.vcf is None):
         parser.print_help()
         exit("ERROR: load needs both a VCF file and a database file\n")
-    if args.anno_type not in ['snpEff', 'VEP', None]:
-        parser.print_help()
-        exit("\nERROR: Unsupported selection for -t\n")
 
     # collect of the the add'l annotation files
     annotations.load_annos()
 
-    if use_scheduler(args):
+    if args.scheduler:
         load_ipython(args)
     elif args.cores > 1:
         load_multicore(args)
@@ -140,7 +137,7 @@ def load_chunks_multicore(grabix_file, args):
         passonly = "--passonly"
 
 
-    submit_command = get_submit_command(args)
+    submit_command = "{cmd}"
     vcf, _ = os.path.splitext(grabix_file)
     chunk_steps = get_chunk_steps(grabix_file, args)
     chunk_vcfs = []
@@ -149,7 +146,7 @@ def load_chunks_multicore(grabix_file, args):
 
     for chunk_num, chunk in chunk_steps:
         start, stop = chunk
-        print "Loading chunk " + str(chunk_num) + "." + ped_file
+        print "Loading chunk " + str(chunk_num) + "."
         gemini_load = gemini_pipe_load_cmd().format(**locals())
         procs.append(subprocess.Popen(submit_command.format(cmd=gemini_load),
                                       shell=True))
@@ -297,13 +294,8 @@ def is_gz_file(fname):
         return False
 
 def get_submit_command(args):
-    if args.lsf_queue:
-        return get_lsf_command(args.lsf_queue)
-    else:
-        return "{cmd}"
+    return "{cmd}"
 
-def get_lsf_command(queue):
-    return "bsub -K -q %s {cmd}" % (queue)
 
 def file_exists(fname):
     """Check if a file exists and is non-empty.
@@ -334,14 +326,7 @@ def combine_dicts(d1, d2):
     return dict(d1.items() + d2.items())
 
 def get_ipython_args(args):
-    if args.lsf_queue:
-        return ("lsf", args.lsf_queue, args.cores)
-    elif args.sge_queue:
-        return ("sge", args.sge_queue, args.cores)
-    elif args.torque_queue:
-        return ("torque", args.torque_queue, args.cores)
-    else:
-        raise ValueError("ipython argument parsing failed for some reason.")
+    return (args.scheduler, args.queue, args.cores)
 
 def print_cmd_not_found_and_exit(cmd):
     sys.stderr.write("Cannot find {cmd}, install it or put it in your "
@@ -349,7 +334,4 @@ def print_cmd_not_found_and_exit(cmd):
     exit(1)
 
 def use_scheduler(args):
-    if any([args.lsf_queue, args.sge_queue, args.torque_queue]):
-        return True
-    else:
-        return False
+    return bool(args.scheduler)
