@@ -9,7 +9,9 @@ import gemini_query
 import \
     gemini_region, gemini_stats, gemini_dump, \
     gemini_annotate, gemini_windower, \
-    gemini_browser, gemini_dbinfo, gemini_merge_chunks, gemini_update
+    gemini_browser, gemini_dbinfo, gemini_merge_chunks, gemini_update, \
+    gemini_amend
+
 import gemini.version
 
 import tool_compound_hets
@@ -91,11 +93,8 @@ def main():
     parser_load.add_argument('-v', dest='vcf',
                              help='The VCF file to be loaded.')
     parser_load.add_argument('-t', dest='anno_type',
-                             default=None, metavar='STRING',
-                             help="The annotations to be used with the input vcf. Options are:\n"
-                             "  snpEff  - Annotations as reported by snpEff.\n"
-                             "  VEP     - Annotations as reported by VEP.\n"
-                             )
+                             default=None, choices=["snpEff", "VEP"],
+                             help="The annotations to be used with the input vcf.")
     parser_load.add_argument('-p', dest='ped_file',
                              help='Sample information file in PED+ format.',
                              default=None)
@@ -118,15 +117,12 @@ def main():
                              default=1,
                              type=int,
                              help="Number of cores to use to load in parallel.")
-    parser_load.add_argument('--lsf-queue',
-                             dest='lsf_queue',
-                             help="Queue name to use for Platform LSF.")
-    parser_load.add_argument('--sge-queue',
-                             dest='sge_queue',
-                             help="Queue name to use for Sun Grid Engine.")
-    parser_load.add_argument('--torque-queue',
-                             dest='torque_queue',
-                             help="Queue name to use for a Torque based scheduler")
+    parser_load.add_argument('--scheduler', dest='scheduler', default=None,
+                             choices=["lsf", "sge", "slurm", "torque"],
+                             action=IPythonAction,
+                             help='Cluster scheduler to use.')
+    parser_load.add_argument('--queue', dest='queue',
+                             default=None, help='Cluster queue to use.')
     parser_load.add_argument('--passonly',
                              dest='passonly',
                              default=False,
@@ -134,6 +130,19 @@ def main():
                              help="Keep only variants that pass all filters.")
 
     parser_load.set_defaults(func=gemini_load.load)
+    #########################################
+    # $ gemini amend
+    #########################################
+    parser_amend = subparsers.add_parser('amend',
+                                         help="Amend an already loaded GEMINI database.")
+    parser_amend.add_argument('db',
+                              metavar='db',
+                              help='The name of the database to be amended.')
+    parser_amend.add_argument('--sample',
+                              metavar='sample',
+                              default=None,
+                              help='New sample information file to load')
+    parser_amend.set_defaults(func=gemini_amend.amend)
 
     #########################################
     # $ gemini load_chunk
@@ -221,6 +230,12 @@ def main():
                               default=False,
                               help=('Add a column of all sample names with a variant to each '
                                     'variant.'))
+    parser_query.add_argument('--show-families',
+                              dest='show_families',
+                              action='store_true',
+                              default=False,
+                              help=('Add a column listing all of the families '
+                                    'with a variant to each variant.'))
     parser_query.add_argument('--family-wise',
                               dest='family_wise',
                               default=False,
@@ -264,6 +279,12 @@ def main():
                               default=None,
                               help=('Restrict query to this region, '
                                     'e.g. chr1:10-20.'))
+    parser_query.add_argument('--carrier-summary-by-phenotype',
+                              dest='carrier_summary',
+                              default=None,
+                              help=('Output columns of counts of carriers and '
+                                    'non-carriers stratified by the given '
+                                    'sample phenotype column'))
 
     parser_query.set_defaults(func=gemini_query.query)
 
@@ -764,6 +785,17 @@ def main():
         if e.errno != 32:  # ignore SIGPIPE
             raise
 
+class IPythonAction(argparse.Action):
+    def __call__(self, parser, args, values, option = None):
+        args.scheduler = values
+        if xor(args.scheduler, args.queue):
+            parser.error("If you are using the IPython parallel loading, you "
+                         "must specify both a scheduler with --scheduler and a "
+                         "queue to use with --queue.")
+
+def xor(arg1, arg2):
+    return bool(arg1) ^ bool(arg2)
+
 
 if __name__ == "__main__":
     main()
